package ca.recrown.islandsurvivalcraft.caching;

import static org.junit.Assert.assertSame;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertTrue;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestInstance;
import org.junit.jupiter.api.TestInstance.Lifecycle;

@TestInstance(Lifecycle.PER_CLASS)
public class CacheTest {
    Cache<Integer, String> integerCache;

    @BeforeAll
    public void setUp() {
        integerCache = new Cache<>(3);
    }

    @AfterEach
    public void cleanUp() {
        integerCache.clearCache();
    }

    @Test
    public void testBasicCaching() {
        String val = integerCache.getValue(0);
        assertTrue(val == null);
        val = "first";
        integerCache.setValue(0, val);
        assertTrue(integerCache.hasValue(0));
        assertSame(val, integerCache.getValue(0));

        val = integerCache.getValue(1);
        assertTrue(val == null);
        val = "second";
        integerCache.setValue(1, val);
        assertTrue(integerCache.hasValue(1));
        assertSame(val, integerCache.getValue(1));
    }

    @Test
    public void testUsageBasedClean() {
        integerCache.setValue(0, "first");
        integerCache.setValue(1, "second");
        integerCache.setValue(2, "third");
        
        assertEquals("first", integerCache.getValue(0));

        integerCache.setValue(3, "fourth");
        
        assertEquals("first", integerCache.getValue(0));
        assertTrue(integerCache.hasValue(1));

        integerCache.setValue(5, "sixth");
        assertFalse(integerCache.hasValue(2));
    }
}